// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "sim.h"





void plot_open(struct device *in)
{
if (in->enable_plot==TRUE)
{
	in->gnuplot = popen("gnuplot -persist","w");
	fprintf(in->gnuplot, "set terminal x11 title 'Solarsim' \n");
	fprintf(in->gnuplot, "clear\n");
	//fprintf(in->gnuplot, "set multiplot\n");
	fprintf(in->gnuplot, "set data style lines\n");
	fflush(in->gnuplot);
}
}

struct timeval last_time;

void plot_now(struct device *in,char *name)
{
struct timeval mytime;

gettimeofday (&mytime, NULL);

if (mytime.tv_sec==last_time.tv_sec)
if ((mytime.tv_usec-last_time.tv_usec)<0.5e6) return;

last_time.tv_sec=mytime.tv_sec;
last_time.tv_usec=mytime.tv_usec;

if (in->enable_plot==TRUE)
{

	if (in->time_on==FALSE)
	{
		fprintf(in->gnuplot, "load '%s'\n",name);
	}else
	{
		fprintf(in->gnuplot, "load 'plottime'\n");
	}
	fflush(in->gnuplot);
}
}

void plot_now_excite(struct device *in)
{
if (in->enable_plot==TRUE)
{

	fprintf(in->gnuplot, "load 'plot_excite'\n");
	fflush(in->gnuplot);
}
}

void plot_replot(struct device *in)
{
if (in->enable_plot==TRUE)
{

	fprintf(in->gnuplot, "replot\n");
	fflush(in->gnuplot);
}
}



void plot_close(struct device *in)
{
if (in->enable_plot==TRUE)
{

	fprintf(in->gnuplot, "quit\n");
	fclose(in->gnuplot);
}
}
